@@ -102,7 +102,7 @@ class Blockchain(util.PrintError):
         return hash_encode(Hash(self.serialize_header(header).decode('hex')))
 
     def path(self):
-        return os.path.join(self.config.path, 'blockchain_headers')
+        return util.get_headers_path(self.config)
 
     def init_headers_file(self):
         filename = self.path()
