@@ -216,24 +216,27 @@ class Commands:
     @command('wp')
     def signtransaction(self, tx, privkey=None):
         """Sign a transaction. The wallet keys will be used unless a private key is provided."""
-        t = Transaction(tx)
+        tx = Transaction(tx)
         if privkey:
             pubkey = bitcoin.public_key_from_private_key(privkey)
-            t.sign({pubkey:privkey})
+            h160 = bitcoin.hash_160(pubkey.decode('hex'))
+            x_pubkey = 'fd' + (chr(0) + h160).encode('hex')
+            tx.sign({x_pubkey:privkey})
         else:
-            self.wallet.sign_transaction(t, self._password)
-        return t.as_dict()
+            self.wallet.sign_transaction(tx, self._password)
+        return tx.as_dict()
 
     @command('')
     def deserialize(self, tx):
         """Deserialize a serialized transaction"""
-        return Transaction(tx).deserialize()
+        tx = Transaction(tx)
+        return tx.deserialize()
 
     @command('n')
-    def broadcast(self, tx, timeout=10):
+    def broadcast(self, tx, timeout=30):
         """Broadcast a transaction to the network. """
-        t = Transaction(tx)
-        return self.network.broadcast(t, timeout)
+        tx = Transaction(tx)
+        return self.network.broadcast(tx, timeout)
 
     @command('')
     def createmultisig(self, num, pubkeys):
@@ -323,8 +326,6 @@ class Commands:
     @command('n')
     def getservers(self):
         """Return the list of available servers"""
-        while not self.network.is_up_to_date():
-            time.sleep(0.1)
         return self.network.get_servers()
 
     @command('')
@@ -614,6 +615,11 @@ class Commands:
         self.network.send([('blockchain.address.subscribe', [address])], callback)
         return True
 
+    @command('wn')
+    def is_synchronized(self):
+        """ return wallet synchronization status """
+        return self.wallet.is_up_to_date()
+
     @command('')
     def help(self):
         # for the python console
@@ -669,12 +675,13 @@ command_options = {
 
 
 # don't use floats because of rounding errors
+from transaction import tx_from_str
 json_loads = lambda x: json.loads(x, parse_float=lambda x: str(Decimal(x)))
 arg_types = {
     'num': int,
     'nbits': int,
     'entropy': long,
-    'tx': json_loads,
+    'tx': tx_from_str,
     'pubkeys': json_loads,
     'inputs': json_loads,
     'outputs': json_loads,
@@ -758,6 +765,8 @@ def get_parser():
     for cmdname in sorted(known_commands.keys()):
         cmd = known_commands[cmdname]
         p = subparsers.add_parser(cmdname, parents=[parent_parser], help=cmd.help, description=cmd.description)
+        if cmdname == 'restore':
+            p.add_argument("-o", "--offline", action="store_true", dest="offline", default=False, help="Run offline")
         #p.set_defaults(func=run_cmdline)
         if cmd.requires_password:
             p.add_argument("-W", "--password", dest="password", default=None, help="password")
