@@ -217,7 +217,7 @@ class Network(util.DaemonThread):
         # to or have an ongoing connection with
         self.interface = None
         self.interfaces = {}
-        self.auto_connect = self.config.get('auto_connect', False)
+        self.auto_connect = self.config.get('auto_connect', True)
         self.connecting = set()
         self.socket_queue = Queue.Queue()
         self.start_network(deserialize_server(self.default_server)[2],
@@ -821,7 +821,7 @@ class Network(util.DaemonThread):
     def get_local_height(self):
         return self.blockchain.height()
 
-    def synchronous_get(self, request, timeout=100000000):
+    def synchronous_get(self, request, timeout=30):
         queue = Queue.Queue()
         self.send([request], queue.put)
         try:
@@ -832,7 +832,7 @@ class Network(util.DaemonThread):
             raise BaseException(r.get('error'))
         return r.get('result')
 
-    def broadcast(self, tx, timeout=10):
+    def broadcast(self, tx, timeout=30):
         tx_hash = tx.hash()
         try:
             out = self.synchronous_get(('blockchain.transaction.broadcast', [str(tx)]), timeout)
